/******************************************************************************
* C Source: Plate List                                                        *
*                                                                             *
* File:                                                                       *
*    platelst.c                                                               *
*                                                                             *
* Description:                                                                *
*    This set of functions performs various manipulations on a linked list    *
*    (see lnkd_lst.c and lnkd_lst.h) of Plates (see plate.c and plate.h),     *
*    each node for an individual plate. (see platelst.h).                     *
*                                                                             *
* Module List:                                                                *
*    Add_Plate_to_List                                                        *
*       Insert a plate into the top of a plate list.                          *
*    Create_Empty_Plate_List                                                  *
*       Create a new empty plate list.                                        *
*    Delete_Plate_List                                                        *
*       Delete the linked list for a plate list, but not the plate data the   *
*       list points to.                                                       *
*    Delete_Plate_List_Data                                                   *
*       Delete the linked list for a plate list, AND the plate data the       *
*       list points to.                                                       *
*    Find_Plate_in_List                                                       *
*       Find plate with given ID in plate list.                               *
*    Make_Plate_List_File_Name                                                *
*       Create full plate list file name                                      *
*    Read_Plate_List                                                          *
*       Read data from plate list on disk.                                    *
*                                                                             *
* History:                                                                    *
*    Created    : 15-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
*                                                                             *
* Copyright (c) 1993, 1994, Association of Universities for Research in       *
* Astronomy, Inc.  All Rights Reserved.                                       *
* Produced under National Aeronautics and Space Administration grant          *
* NAG W-2166.                                                                 *
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "platelst.h"

#include "astronmy.h"
#include "config.h"
#include "filenams.h"
#include "generic.h"
#include "lnkd_lst.h"
#include "plate.h"

/******************************************************************************
* Module:                                                                     *
*    Add_Plate_to_List                                                        *
*                                                                             *
* File:                                                                       *
*    platelst.c                                                               *
*                                                                             *
* Description:                                                                *
*    Adds a plate to the top of a plate list.                                 *
*                                                                             *
* Return Value:                                                               *
*    none                                                                     *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       List                                                                  *
*          PLATE_LIST, Reference                                              *
*             The plate list to which we're adding the plate.                 *
*       Plate                                                                 *
*          PLATE, Reference                                                   *
*             The plate we're adding to the plate list.                       *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 15-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/*********************************
* Add_Plate_to_List declarations *
*********************************/
void       Add_Plate_to_List( List, Plate )
PLATE_LIST List ;
PLATE      Plate;
{
/******************************************
* Add_Plate_to_List begin executable code *
******************************************/
   Insert_Linked_List_Data( List, Plate );

   return;
}
/****************************************
* Add_Plate_to_List end executable code *
****************************************/

/******************************************************************************
* Module:                                                                     *
*    Create_Empty_Plate_List                                                  *
*                                                                             *
* File:                                                                       *
*    platelst.c                                                               *
*                                                                             *
* Description:                                                                *
*    Create a new empty plate list, allocating memory for the header.         *
*                                                                             *
* Return Value:                                                               *
*    PLATE_LIST: The newly created plate.                                     *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       none                                                                  *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 15-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/***************************************
* Create_Empty_Plate_List declarations *
***************************************/
PLATE_LIST Create_Empty_Plate_List()
{
/************************************************
* Create_Empty_Plate_List begin executable code *
************************************************/
   return ( PLATE_LIST )Create_Empty_Linked_List();
}
/**********************************************
* Create_Empty_Plate_List end executable code *
**********************************************/

/******************************************************************************
* Module:                                                                     *
*    Delete_Plate_List                                                        *
*                                                                             *
* File:                                                                       *
*    platelst.c                                                               *
*                                                                             *
* Description:                                                                *
*    Free memory allocated for plate list, but not the data it points to.     *
*                                                                             *
* Return Value:                                                               *
*    none                                                                     *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       List                                                                  *
*          PLATE_LIST, Reference                                              *
*             The plate list we're deleting from memory.                      *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 15-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/*********************************
* Delete_Plate_List declarations *
*********************************/
void       Delete_Plate_List( List )
PLATE_LIST List;
{
/******************************************
* Delete_Plate_List begin executable code *
******************************************/
   Delete_Linked_List( List );

   return;
}
/****************************************
* Delete_Plate_List end executable code *
****************************************/

/******************************************************************************
* Module:                                                                     *
*    Delete_Plate_List_Data                                                   *
*                                                                             *
* File:                                                                       *
*    platelst.c                                                               *
*                                                                             *
* Description:                                                                *
*    Free memory allocated for plate list, but AND the data it points to.     *
*                                                                             *
* Return Value:                                                               *
*    none                                                                     *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       List                                                                  *
*          PLATE_LIST, Reference                                              *
*             The plate list we're deleting from memory.                      *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 15-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/**************************************
* Delete_Plate_List_Data declarations *
**************************************/
void       Delete_Plate_List_Data( List )
PLATE_LIST List;
{
/***********************************************
* Delete_Plate_List_Data begin executable code *
***********************************************/
   Delete_Linked_List_Data( List, Delete_Plate );

   return;
}
/*********************************************
* Delete_Plate_List_Data end executable code *
*********************************************/

/******************************************************************************
* Module:                                                                     *
*    Find_Plate_in_List                                                       *
*                                                                             *
* File:                                                                       *
*    platelst.c                                                               *
*                                                                             *
* Description:                                                                *
*    Find plate with given ID in plate list.                                  *
*                                                                             *
* Return Value:                                                               *
*    PLATE: Plate data found in plate list. NULL is returned if the plate is  *
*    not found.                                                               *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       List                                                                  *
*          PLATE_LIST, Reference                                              *
*             The plate list we're deleting from memory.                      *
*       Plate_Id                                                              *
*          STRING, Reference                                                  *
*             Id of plate to find in the plate list.                          *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 25-OCT-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/**********************************
* Find_Plate_in_List declarations *
**********************************/
PLATE      Find_Plate_in_List( List, Plate_Id )
PLATE_LIST List;
STRING     Plate_Id;
{
   PLATE Plate; /* Plate found in plate list */

/*******************************************
* Find_Plate_in_List begin executable code *
*******************************************/
   return ( PLATE )Find_Linked_List_Data( List, Plate_Id, strcmp);
}
/*****************************************
* Find_Plate_in_List end executable code *
*****************************************/

/******************************************************************************
* Module:                                                                     *
*    Make_Plate_List_File_Name                                                *
*                                                                             *
* File:                                                                       *
*    platelst.c                                                               *
*                                                                             *
* Description:                                                                *
*    Create full file name for plate list from given directory, and the       *
*    symbolic constants PLATE_LIST_FILE or DEFAULT_PLATE_LIST_FILE.           *
*                                                                             *
* Return Value:                                                               *
*    STRING: Name of plate list file.                                         *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       Directory                                                             *
*          STRING, Reference                                                  *
*             Name of directory where plate list file resides.                *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 19-NOV-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/*****************************************
* Make_Plate_List_File_Name declarations *
*****************************************/
STRING Make_Plate_List_File_Name(Directory)
STRING Directory;
{
   STRING Plate_List_File_Name     ; /* Plate for just the file name */
   STRING Full_Plate_List_File_Name; /* File name with path          */

/**************************************************
* Make_Plate_List_File_Name begin executable code *
**************************************************/
   Plate_List_File_Name = Translate_Environment_Variable( PLATE_LIST_FILE );

   if ( Plate_List_File_Name == NULL )
      Plate_List_File_Name = DEFAULT_PLATE_LIST_FILE;

   if (!Set_FileNames_Case(Directory, Plate_List_File_Name))
   {
      fprintf(stderr,"\nCould not find copyright file in header directory!\n");
      fprintf(stderr, "Aborting GetImage!!!\n\n");
      exit(EXIT_FAILURE);
   }

   Full_Plate_List_File_Name =
         Build_Full_File_Name( Directory, "", Plate_List_File_Name );

   free(Plate_List_File_Name);

   Plate_List_File_Name =

/***********************************
* Make_Plate_List_File_Name page 2 *
***********************************/

         Set_FileNames_Version(Directory, Full_Plate_List_File_Name);

   if (Plate_List_File_Name != Full_Plate_List_File_Name)
      if (Plate_List_File_Name != NULL)
         free(Full_Plate_List_File_Name);
      else {
         fprintf(stderr,
               "\nCould not find copyright file in header directory!\n");
         fprintf(stderr, "Aborting GetImage!!!\n\n");
         exit(EXIT_FAILURE);
      }

   Full_Plate_List_File_Name = Plate_List_File_Name;

   return Full_Plate_List_File_Name;
}
/************************************************
* Make_Plate_List_File_Name end executable code *
************************************************/

/******************************************************************************
* Module:                                                                     *
*    Read_Plate_List                                                          *
*                                                                             *
* File:                                                                       *
*    platelst.c                                                               *
*                                                                             *
* Description:                                                                *
*    Read data for plates in plate list on disk into internal plate list.     *
*                                                                             *
* Return Value:                                                               *
*    PLATE_LIST: New internal plate list created from plate list on disk.     *
*                                                                             *
* Arguments;                                                                  *
*                                                                             *
*    Input Arguments:                                                         *
*       Directory                                                             *
*          STRING, Reference                                                  *
*             Name of the directory where the plate list file resides.        *
*       Plate_File                                                            *
*             Name of the plate list file in Directory                        *
*          STRING, Reference                                                  *
*    Input/Output Arguments:                                                  *
*       none                                                                  *
*    Output Arguments:                                                        *
*       none                                                                  *
*                                                                             *
* History:                                                                    *
*    Created    : 15-JUN-93 by J. Doggett,                                    *
*       for the Association of Universities for Research in Astronomy, Inc.   *
*          at the Space Telescope Science Institute,                          *
*       to facilitate the accessing of data from the CD-ROM set of the        *
*          Digitized Sky Survey.                                              *
*    Delivered  : 01-MAR-94 by J. Doggett,                                    *
*       GetImage Version 1.0                                                  *
******************************************************************************/
/*******************************
* Read_Plate_List declarations *
*******************************/
PLATE_LIST Read_Plate_List( Plate_List_File_Name )
STRING     Plate_List_File_Name;
{
   FILE       *Plate_List_File; /* Pointer to plate list file    */
   PLATE_LIST  Plate_List     ; /* New plate list read from disk */
   PLATE Plate     ; /* Place to store plate data read from file */

/****************************************
* Read_Plate_List begin exectuable code *
****************************************/
//   printf( "\nPlease standby; reading plate list:\n\n%s...\n",
//                                                 Plate_List_File_Name );
   Plate_List_File = fopen( Plate_List_File_Name, "r" );
   if ( Plate_List_File == NULL )
   {
      fprintf( stderr, "\nCould not open plate list file %s.\n\n",
                                                        Plate_List_File_Name );
      exit( EXIT_FAILURE );
   }

/* Read plates into plate list until the file is exhausted */
   Plate_List = Create_Empty_Plate_List();

   Plate = Read_Plate_Info(Plate_List_File);
   while ((int)Plate != EOF )

/*************************
* Read_Plate_List page 2 *
*************************/

   {
      if ((int)Plate == (int)NULL)
      {
         fprintf(stderr, "\nError reading plate list file %s.\n",
		 Plate_List_File_Name);
	 fprintf(stderr, "Aborting...\n\n");
	 exit( EXIT_FAILURE );
      }

      Add_Plate_to_List( Plate_List, Plate );

      Plate = Read_Plate_Info(Plate_List_File);
   }

   fclose( Plate_List_File );

   return Plate_List;
}
/**************************************
* Read_Plate_List end exectuable code *
**************************************/
